/*
 * gmg12864_r9ofg.c
 *
 * Autor: R9OFG.RU
 * Ver: 1.2
 * Data: 09-10-2023
 *
 *    GMG12864-06D
 *
 *    c :
 * https://www.youtube.com/watch?v=ajEqZN5s5xc
 * https://narodstream.ru/stm-urok-37-displej-tft-240x320-8bit-chast-1/
 * https://hubstub.ru/display/126-vyvod-simvolov-i-strok-na-tft-displey-na-primere-ili9341.html
 */

#include <string.h>

#include "gmg12864_r9ofg.h"
#include "font_tahoma_8_prop.h"

/*  */

uint8_t Buffer_Frame[1024] 	= {0,0};	// 
uint8_t chSpacing			= 0;		//   px
uint8_t lastPxString_X		= 0;		//     X

/*  */

extern SPI_HandleTypeDef hspi1;			//  SPI

/*   */

/*
 *     
 *  sendCommand - 8  
 */
static void LCD_Send_Command(uint8_t sendCommand)
{
	DC_set();
	HAL_SPI_Transmit(&hspi1, &sendCommand, 1, HAL_MAX_DELAY);
	while (HAL_SPI_GetState(&hspi1) != HAL_SPI_STATE_READY) {};
	DC_reset();
}

/*
 *     
 *  sendData - 8  
 */
static void LCD_Send_Data(uint8_t sendData)
{
	HAL_SPI_Transmit(&hspi1, &sendData, 1, HAL_MAX_DELAY);
	while (HAL_SPI_GetState(&hspi1) != HAL_SPI_STATE_READY) {};
}

/*     */
void LCD_Clean_Buffer_Frame(void)
{
	memset(Buffer_Frame, 0x00, sizeof(Buffer_Frame));
}

/*       ( ) */
void LCD_Update(void)
{
	CS_set();
	for (uint8_t y=0;y<8;y++)
	{
		LCD_Set_xPos(0);
		LCD_Set_yPos(y);
		for (uint8_t x=0;x<LCD_Width;x++)
		{
			LCD_Send_Data(Buffer_Frame[x + LCD_Width * y]);
		}
	}
	CS_reset();
}

/*    */
void LCD_Init(void)
{
	CS_set();
	RST_set();
	HAL_Delay(10);
	RST_reset();
	HAL_Delay(10);
	//   ( 1/7  1/9 )     -
	LCD_Send_Command(0xA2);
	HAL_Delay(1);
	//       
	LCD_Send_Command(0xA0); //ADC selection(SEG0->SEG128)
	LCD_Send_Command(0xC8); //SHL selection(COM0->COM64)
	HAL_Delay(1);
	//     7 ( 0..7)
	LCD_Send_Command(0x20 | 0x7);
	HAL_Delay(1);
	// ,     ( 0..7)
	LCD_Send_Command(0x28 | 0x7);
	HAL_Delay(1);
	//  
	LCD_Send_Command(0x81);
	LCD_Send_Command(15); // . 15-19 ,  63
	HAL_Delay(1);
	LCD_Send_Command(0x40);
	HAL_Delay(1);
	// 
	LCD_Send_Command(0xAF);
	//  (0xA6 - nomal, 0xA7 - revers)
	LCD_Send_Command(0xA6);
	HAL_Delay(1);
	CS_reset();

	//  
	LCD_Clean_Buffer_Frame();
	// 
	LCD_Update();
}

/*    */
void LCD_Draw_Pixel(uint8_t xPos, uint8_t yPos, uint8_t pxColor)
{
	if ((xPos >= LCD_Width)||(xPos < 0)||(yPos >= LCD_Height)||(yPos < 0))
	{
		//   x  y      
		return;
	}
	else
	{
		uint16_t arrayPos = xPos + ((yPos/8)*LCD_Width);

		//  
		if (pxColor)
		{
			Buffer_Frame[arrayPos] |= 1 << (yPos % 8);
		}
		else
		{
			Buffer_Frame[arrayPos] &= 0xFF ^ 1 << (yPos % 8);
		}
	}
}

/*        */
void LCD_Inversion_Pos_Buffer_Frame(uint16_t xPos_Start, uint16_t xPos_End)
{
	for (; xPos_Start < xPos_End; xPos_Start++)
	{
		Buffer_Frame[xPos_Start] = ~(Buffer_Frame[xPos_Start]);
	}
}

/*    */

/*      */
void LCD_Draw_Char_1_Byte(uint8_t xPos, uint8_t yPos, char Symbol, uint8_t Inversion)
{
	/*
	 *     "h"    matrixFont
	 * https://habr.com/ru/articles/575332/
	 *  :
	 *  ,   ,  , , HEX, , 8(uint8_t, unsigned char), 99
	 *      , ..     8 
	 */

	/*  ASCII     32...255,     */
	if ((Symbol >= 32) && (Symbol <= 255))
	{
		uint8_t pxChView;
		uint8_t pxBkView;

		///  
		if (!(Inversion & ChInvers))
		{
			pxChView = pxView_On;
			pxBkView = pxView_Off;
		}
		else
		{
			pxChView = pxView_Off;
			pxBkView = pxView_On;
		}

		//   + 1px    
		uint8_t chWidth = font_tahoma_8[(Symbol - 0x20) * (1 + FONT_TAHOMA_8_CHAR_HEIGHT)] + 1;
		//   
		chSpacing = chWidth;

		//   
		for (uint8_t y = 1; y <= FONT_TAHOMA_8_CHAR_HEIGHT; y++) //    
		{
			for (uint8_t x = 0; x < chWidth; x++) //   
			{
				/*
				 *        
				 *         1   0
				 * 0x20  32
				 */

				if (font_tahoma_8[((Symbol - 0x20) * (1 + FONT_TAHOMA_8_CHAR_HEIGHT)) + y] >> (7 - x) & 0x01)
				{
					LCD_Draw_Pixel(xPos + x, yPos + y, pxChView);
				}
				else
				{
					LCD_Draw_Pixel(xPos + x, yPos + y, pxBkView);
				}
			}

			/*
			 *            
			 *        1px
			 */
			if (chWidth < FONT_TAHOMA_8_CHAR_WIDTH)
			{
				for (uint8_t n = chWidth; n < (FONT_TAHOMA_8_CHAR_WIDTH + 1); n++)
				{
					if (!(Inversion & ChInvers))
					{
						LCD_Draw_Pixel(xPos + n, yPos + y, pxBkView);
					}
					else
					{
						LCD_Draw_Pixel(xPos + n, yPos + y, pxChView);
					}
				}
			}
		}
	}
}

/*      */
void LCD_Output_String(uint8_t xPos, uint8_t yPos, char *String, uint8_t setChSpacing, uint8_t Inversion)
{
	/*
	 *    font_tahoma_8
	 *  :
	 * xPos			-   X
	 * yPos			-   Y
	 * *String		-   
	 * setChSpacing	-    px
	 * Inversion	- / 
	 */

	//  
	while(*String != '\n')
	{
		//           ,
		//  ,     
		/*if((xPos + 8) > LCD_Width)
		{
			xPos = xPos_Start;
			yPos = yPos + 10;
		}*/

		//   
		LCD_Draw_Char_1_Byte(xPos, yPos, *String, Inversion);

		//     
		xPos += chSpacing - 1 + setChSpacing;

		//      
		String++;
	}

	//    X
	lastPxString_X = xPos;
}

/*   */

/*    */
void LCD_Draw_Line(uint8_t xPos_Start, uint8_t yPos_Start, uint8_t xPos_End, uint8_t yPos_End, uint8_t pxColor)
{
	int dx = (xPos_End >= xPos_Start) ? xPos_End - xPos_Start : xPos_Start - xPos_End;
	int dy = (yPos_End >= yPos_Start) ? yPos_End - yPos_Start : yPos_Start - yPos_End;
	int sx = (xPos_Start < xPos_End) ? 1 : -1;
	int sy = (yPos_Start < yPos_End) ? 1 : -1;
	int err = dx - dy;

	for (;;)
	{
		LCD_Draw_Pixel(xPos_Start, yPos_Start, pxColor);
		if (xPos_Start == xPos_End && yPos_Start == yPos_End)
			break;
		int e2 = err + err;
		if (e2 > -dy)
		{
			err -= dy;
			xPos_Start += sx;
		}
		if (e2 < dx)
		{
			err += dx;
			yPos_Start += sy;
		}
	}
}

/*     */
void LCD_Draw_Rectangle(uint8_t xPos_Start, uint8_t yPos_Start, uint8_t rectangle_Width, uint8_t rectangle_Height, uint8_t pxColor)
{
	/*    */
	//  
	LCD_Draw_Line(xPos_Start, yPos_Start, xPos_Start, yPos_Start + rectangle_Height, pxColor);
	// 
	LCD_Draw_Line(xPos_Start, yPos_Start, xPos_Start + rectangle_Width, yPos_Start, pxColor);
	//  
	LCD_Draw_Line(xPos_Start + rectangle_Width, yPos_Start, xPos_Start + rectangle_Width, yPos_Start + rectangle_Height, pxColor);
	// 
	LCD_Draw_Line(xPos_Start, yPos_Start + rectangle_Height, xPos_Start + rectangle_Width, yPos_Start + rectangle_Height, pxColor);
}

/*     */
void LCD_Draw_Rectangle_Filled(uint8_t xPos_Start, uint8_t yPos_Start, uint8_t rectangle_Width, uint8_t rectangle_Height, uint8_t pxColor)
{
	for (uint8_t i = 0; i <= rectangle_Height; i++)
	{
		LCD_Draw_Line(xPos_Start, yPos_Start + i, xPos_Start + rectangle_Width, yPos_Start + i, pxColor);
	}
}
