/*
 * ina226_r9ofg.c
 *
 *  Created on: Dec 1, 2023
 *  Author: r9ofg.ru
 *
 *        VBUS!!!
 */

/*  */

#include <stdio.h>
#include "ina226_r9ofg.h"

/*   */
uint8_t flag_ina226 		= 0;		//     I2C
uint8_t INA226_I2C_ADDRESS  = 0;		//    I2C
float  vbus_calibrate		= 124.5;	//   

/*    */

extern I2C_HandleTypeDef hi2c1;

/*  */

/*    INA226   I2C */
void INA226_search_on_i2c_bus()
{
	/*
	 *    Datasheet INA226:
	 * Table 7. INA226 Address Pins and
	 * Slave Addresses
	 * A1 A0 SLAVE ADDRESS
	 * ------------------------------
	 *    |   VAR   |   BIN   | DEC |
	 * ------------------------------
	 * 1  | GND GND | 1000000 | 64  |
	 * 2  | GND VS+ | 1000001 | 65  |
	 * 3  | GND SDA | 1000010 | 66  |
	 * 4  | GND SCL | 1000011 | 67  |
	 * 5  | VS+ GND | 1000100 | 68  |
	 * 6  | VS+ VS+ | 1000101 | 69  |
	 * 7  | VS+ SDA | 1000110 | 70  |
	 * 8  | VS+ SCL | 1000111 | 71  |
	 * 9  | SDA GND | 1001000 | 72  |
	 * 10 | SDA VS+ | 1001001 | 73  |
	 * 11 | SDA SDA | 1001010 | 74  |
	 * 12 | SDA SCL | 1001011 | 75  |
	 * 13 | SCL GND | 1001100 | 76  |
	 * 14 | SCL VS+ | 1001101 | 77  |
	 * 15 | SCL SDA | 1001110 | 78  |
	 * 16 | SCL SCL | 1001111 | 79  |
	 * ------------------------------
	 *
	 *      ,
	 *   ,  ,   INA226
	 * ..        i2c.
	 */

	HAL_StatusTypeDef ret;

	for(uint16_t i = 64; i < 80; i++)
	{
	    ret = HAL_I2C_IsDeviceReady(&hi2c1, (uint16_t)(i << 1), 4, 100);
	    if(ret == HAL_OK)
	    {
	        //    . datasheet
	    	if((i >= 64)&&(i <= 79))
	        {
	        	//  
	        	INA226_I2C_ADDRESS = i;
	        	//      I2C
	        	flag_ina226 = 1;
	        }
	    	else
	    	{
	    		//      I2C,
	    		//          
	    		flag_ina226 = 0;
	    	}return;
	    }
	    else if(ret != HAL_OK)
	    {
	    	//      I2C,       
	    	flag_ina226 = 0;
	    }
	}
}

/*    INA226 */
void INA226_init()
{
	/*
	 * !!!         VBUS,   INA226  !
	 *    !
	 */

	/*uint8_t  Data[3];
	uint8_t  MSB;
	uint8_t  LSB;
	uint16_t CONFIGURATION_REGISTER;*/

	//    I2C,     
	INA226_search_on_i2c_bus();

	/*
	 *      datasheet
	 * 0xC000 - 
	 * 1	  - Averaging Mode
	 * 7	  - Bus Voltage Conversion Time
	 * 6	  - Operating Mode
	 */
	/*CONFIGURATION_REGISTER = 0xC000|(1 << 9)|(6 << 6)|6;

	MSB = (uint8_t)(CONFIGURATION_REGISTER >> 8);
	LSB = (uint8_t)(CONFIGURATION_REGISTER & 0xFF);

	Data[0] = 0;	//Configuration Register
	Data[1] = MSB;
	Data[2] = LSB;

	//     INA226
	HAL_I2C_Master_Transmit (&hi2c1, (INA226_I2C_ADDRESS|(1<<0)<<1), (uint8_t*)Data, 3, 100);*/
}

/*       VBUS  INA226 */
float INA226_vbus(void)
{
	uint8_t  Data[3];
	if(flag_ina226)
	{
		Data[0] = 2; //Bus Voltage

		//INA226_I2C_ADDRESS - 7   +  -0/-1
		HAL_I2C_Master_Transmit(&hi2c1, 0b10000001/*(INA226_I2C_ADDRESS|(1<<0)<<1)*/, (uint8_t*)Data, 1, 10);
		HAL_I2C_Master_Receive(&hi2c1, 0b10000001/*(INA226_I2C_ADDRESS<<1)*/, (uint8_t*)Data, 2, 10);
		return (float)((uint32_t)((Data[0]<<8)|Data[1]) * vbus_calibrate / 100000.0f); //vbus_calibrate - VBUS voltage LSB . datasheet
	}
	else
	{
		return -1;
	}
}
