/* USER CODE BEGIN Header */
/**
  ******************************************************************************
  * @file           : main.c
  * @brief          : Main program body
  ******************************************************************************
  * @attention
  *
  * Copyright (c) 2023 STMicroelectronics.
  * All rights reserved.
  *
  * This software is licensed under terms that can be found in the LICENSE file
  * in the root directory of this software component.
  * If no LICENSE file comes with this software, it is provided AS-IS.
  *
  ******************************************************************************
  */
/* USER CODE END Header */
/* Includes ------------------------------------------------------------------*/
#include "main.h"

/* Private includes ----------------------------------------------------------*/
/* USER CODE BEGIN Includes */
#include <stdio.h>
//#include <stdbool.h>

#include "gmg12864_r9ofg.h"
#include "_program_mode.h"
#include "ina226_r9ofg.h"

/* USER CODE END Includes */

/* Private typedef -----------------------------------------------------------*/
/* USER CODE BEGIN PTD */

/* USER CODE END PTD */

/* Private define ------------------------------------------------------------*/
/* USER CODE BEGIN PD */

#define START_ADC_PERIOD_MS 			1						//       -1,    
#define CALC_ADC_PERIOD_MS				200						//          -1,    
#define MEDIAN_NUM_READ					10						//      -1

/* USER CODE END PD */

/* Private macro -------------------------------------------------------------*/
/* USER CODE BEGIN PM */

/* USER CODE END PM */

/* Private variables ---------------------------------------------------------*/
ADC_HandleTypeDef hadc1;
DMA_HandleTypeDef hdma_adc1;

I2C_HandleTypeDef hi2c1;

SPI_HandleTypeDef hspi1;

/* USER CODE BEGIN PV */

uint8_t  end_adc_flag					= 1;					//   -1
uint8_t  end_adc_filter_flag 			= 0;					//     -1
uint32_t start_adc_cnt 					= START_ADC_PERIOD_MS;	//        -1
uint32_t calc_adc_cnt 					= CALC_ADC_PERIOD_MS;	//        -1
uint32_t adc_samples_cnt				= 0;					//      -1

uint16_t adc_values[ADC_CHANNELS_NUM]	= {0,};					//    -1
uint16_t adc_sum[ADC_CHANNELS_NUM-1]	= {0,};					//    -1  
uint16_t adc_out[ADC_CHANNELS_NUM-1]	= {0,};					//    -1   

extern PROG_STATE 		PROG_state;								//   
extern volatile uint8_t flag_ptt_on;							//   PTT
extern volatile uint8_t att_data;								//   PE4302

/* USER CODE END PV */

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config(void);
static void MX_GPIO_Init(void);
static void MX_DMA_Init(void);
static void MX_ADC1_Init(void);
static void MX_I2C1_Init(void);
static void MX_SPI1_Init(void);
/* USER CODE BEGIN PFP */

/* USER CODE END PFP */

/* Private user code ---------------------------------------------------------*/
/* USER CODE BEGIN 0 */

/* callback   c PTT */
void HAL_GPIO_EXTI_Callback(uint16_t GPIO_Pin)
{
	static volatile uint8_t tx_att_data = 0;

	if(GPIO_Pin == PTT_ON_Pin)
	{
		if(HAL_GPIO_ReadPin(PTT_ON_GPIO_Port, PTT_ON_Pin))
		{
			//PTT 
			flag_ptt_on = 1;
			//   BluPill
			LED_PTT_set();

			//    
			att_data = tx_att_data;
			PE4302_change_att(1);
		}
		else
		{
			//PTT 
			flag_ptt_on = 0;
			//   BluPill
			LED_PTT_reset();

			//         0
			tx_att_data = att_data;
			att_data = 0;
			PE4302_change_att(1);
			att_data = tx_att_data;
		}
	}
}

/*    */
uint16_t findMedianN_optim_mod(uint16_t adc_channel, uint16_t newVal)
{
	/*
	 *   mod AlexGyver
	 * https://alexgyver.ru/lessons/filters/
	 *
	 * mod R9OFG.RU -      ,
	 * ..        -1
	 *
	 */

	static uint16_t buffer[ADC_CHANNELS_NUM-1][MEDIAN_NUM_READ];  //  
	static uint16_t count = 0;
	buffer[adc_channel][count] = newVal;
	if((count < MEDIAN_NUM_READ - 1)&&(buffer[adc_channel][count] > buffer[adc_channel][count + 1]))
	{
		for (int i = count; i < MEDIAN_NUM_READ - 1; i++)
		{
	    	if (buffer[adc_channel][i] > buffer[adc_channel][i + 1])
	    	{
	    		uint16_t buff = buffer[adc_channel][i];
	    		buffer[adc_channel][i] = buffer[adc_channel][i + 1];
	    		buffer[adc_channel][i + 1] = buff;
	    	}
	    }
	}
	else
	{
		if((count > 0)&&(buffer[adc_channel][count - 1] > buffer[adc_channel][count]))
		{
	    	for (int i = count; i > 0; i--)
	    	{
	    		if (buffer[adc_channel][i] < buffer[adc_channel][i - 1])
	    		{
	    			uint16_t buff = buffer[adc_channel][i];
	    			buffer[adc_channel][i] = buffer[adc_channel][i - 1];
	    			buffer[adc_channel][i - 1] = buff;
	    		}
	    	}
	    }
	}
	if (++count >= MEDIAN_NUM_READ) count = 0;
	return buffer[adc_channel][(int)MEDIAN_NUM_READ / 2];
}

/*     -1 */
void ADC_1_start_conversion(void)
{
	/*
	 * MicroTechnics
	 * https://microtechnics.ru/filtracziya-i-usrednenie-dannyh-aczp/
	 *
	 *  ,     startAdcCnt  calcAdcCnt  ,
	 * ,      ,   .
	 *    ,      .           SysTick_Handler,
	 *    ,      .        stm32f1xx_it.c:
	 *
	 * -USER CODE BEGIN EV-
	 * extern uint32_t startAdcCnt;
	 * extern uint32_t calcAdcCnt;
	 * -USER CODE END EV-
	 *
	 * -USER CODE BEGIN SysTick_IRQn 1-
	 * if (startAdcCnt > 0)
	 * {
	 * 	  startAdcCnt--;
	 * }
	 *
	 * if (calcAdcCnt > 0)
	 * {
	 *    calcAdcCnt--;
	 * }
	 * -USER CODE END SysTick_IRQn 1-
	 */

	if(start_adc_cnt == 0)
    {
		if (end_adc_flag == 1)
		{
			//    -1
			end_adc_flag = 0;
			//    -1
			start_adc_cnt = START_ADC_PERIOD_MS;
			//  
			HAL_ADC_Start_DMA(&hadc1, (uint32_t*)&adc_values, ADC_CHANNELS_NUM);
		}
    }

    if (calc_adc_cnt == 0)
    {
    	//    -1
    	for(uint8_t i = 0; i < ADC_CHANNELS_NUM-1; i++)
		{
    		//       2-4 -1
    		adc_out[i] = findMedianN_optim_mod(i, adc_sum[i] / adc_samples_cnt); //  ,   
    		//adc_out[i] = adc_sum[i] / adc_samples_cnt; //  
    		//   
			adc_sum[i] = 0;
		}

    	//    
    	adc_samples_cnt = 0;
    	//   
    	calc_adc_cnt = CALC_ADC_PERIOD_MS;
    	//     -1
    	end_adc_filter_flag = 1;
    }
}

/* callback-   */
void HAL_ADC_ConvCpltCallback(ADC_HandleTypeDef* hadc)
{
	if(hadc->Instance == ADC1)
	{
		//    -1   2-4
		for(uint8_t i = 0; i < ADC_CHANNELS_NUM-1; i++)
		{
			//   
			adc_sum[i] += adc_values[i+1];
			//  
			adc_values[i+1] = 0;
		}
		//    
		adc_samples_cnt++;
		//    -1
		end_adc_flag = 1;
	}
}

/* USER CODE END 0 */

/**
  * @brief  The application entry point.
  * @retval int
  */
int main(void)
{
  /* USER CODE BEGIN 1 */

  /* USER CODE END 1 */

  /* MCU Configuration--------------------------------------------------------*/

  /* Reset of all peripherals, Initializes the Flash interface and the Systick. */
  HAL_Init();

  /* USER CODE BEGIN Init */

  /* USER CODE END Init */

  /* Configure the system clock */
  SystemClock_Config();

  /* USER CODE BEGIN SysInit */

  /* USER CODE END SysInit */

  /* Initialize all configured peripherals */
  MX_GPIO_Init();
  MX_DMA_Init();
  MX_ADC1_Init();
  MX_I2C1_Init();
  MX_SPI1_Init();
  /* USER CODE BEGIN 2 */

  //    PE4301
  PE4302_change_att(0);

  //   PTT
  HAL_GPIO_EXTI_Callback(PTT_ON_Pin);

  // 
  READ_setup_pa();

  // 
  LPF_change_band();

  // -1
  HAL_ADCEx_Calibration_Start(&hadc1);
  //  -1
  ADC_1_start_conversion();

  //   
  PROG_state = PROG_STATE_WORK;

  // 
  LCD_Init();

  //  INA226
  INA226_init();

  // 
  //Splash();

  /* USER CODE END 2 */

  /* Infinite loop */
  /* USER CODE BEGIN WHILE */
  while (1)
  {
	//  
	PROG_mode();

    /* USER CODE END WHILE */

    /* USER CODE BEGIN 3 */
  }
  /* USER CODE END 3 */
}

/**
  * @brief System Clock Configuration
  * @retval None
  */
void SystemClock_Config(void)
{
  RCC_OscInitTypeDef RCC_OscInitStruct = {0};
  RCC_ClkInitTypeDef RCC_ClkInitStruct = {0};
  RCC_PeriphCLKInitTypeDef PeriphClkInit = {0};

  /** Initializes the RCC Oscillators according to the specified parameters
  * in the RCC_OscInitTypeDef structure.
  */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSE;
  RCC_OscInitStruct.HSEState = RCC_HSE_ON;
  RCC_OscInitStruct.HSEPredivValue = RCC_HSE_PREDIV_DIV1;
  RCC_OscInitStruct.HSIState = RCC_HSI_ON;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
  RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSE;
  RCC_OscInitStruct.PLL.PLLMUL = RCC_PLL_MUL9;
  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    Error_Handler();
  }

  /** Initializes the CPU, AHB and APB buses clocks
  */
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV2;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV1;

  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_2) != HAL_OK)
  {
    Error_Handler();
  }
  PeriphClkInit.PeriphClockSelection = RCC_PERIPHCLK_ADC;
  PeriphClkInit.AdcClockSelection = RCC_ADCPCLK2_DIV6;
  if (HAL_RCCEx_PeriphCLKConfig(&PeriphClkInit) != HAL_OK)
  {
    Error_Handler();
  }
}

/**
  * @brief ADC1 Initialization Function
  * @param None
  * @retval None
  */
static void MX_ADC1_Init(void)
{

  /* USER CODE BEGIN ADC1_Init 0 */

  /* USER CODE END ADC1_Init 0 */

  ADC_ChannelConfTypeDef sConfig = {0};

  /* USER CODE BEGIN ADC1_Init 1 */

  /* USER CODE END ADC1_Init 1 */

  /** Common config
  */
  hadc1.Instance = ADC1;
  hadc1.Init.ScanConvMode = ADC_SCAN_ENABLE;
  hadc1.Init.ContinuousConvMode = DISABLE;
  hadc1.Init.DiscontinuousConvMode = DISABLE;
  hadc1.Init.ExternalTrigConv = ADC_SOFTWARE_START;
  hadc1.Init.DataAlign = ADC_DATAALIGN_RIGHT;
  hadc1.Init.NbrOfConversion = 4;
  if (HAL_ADC_Init(&hadc1) != HAL_OK)
  {
    Error_Handler();
  }

  /** Configure Regular Channel
  */
  sConfig.Channel = ADC_CHANNEL_1;
  sConfig.Rank = ADC_REGULAR_RANK_1;
  sConfig.SamplingTime = ADC_SAMPLETIME_41CYCLES_5;
  if (HAL_ADC_ConfigChannel(&hadc1, &sConfig) != HAL_OK)
  {
    Error_Handler();
  }

  /** Configure Regular Channel
  */
  sConfig.Channel = ADC_CHANNEL_2;
  sConfig.Rank = ADC_REGULAR_RANK_2;
  if (HAL_ADC_ConfigChannel(&hadc1, &sConfig) != HAL_OK)
  {
    Error_Handler();
  }

  /** Configure Regular Channel
  */
  sConfig.Channel = ADC_CHANNEL_3;
  sConfig.Rank = ADC_REGULAR_RANK_3;
  if (HAL_ADC_ConfigChannel(&hadc1, &sConfig) != HAL_OK)
  {
    Error_Handler();
  }

  /** Configure Regular Channel
  */
  sConfig.Channel = ADC_CHANNEL_4;
  sConfig.Rank = ADC_REGULAR_RANK_4;
  if (HAL_ADC_ConfigChannel(&hadc1, &sConfig) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN ADC1_Init 2 */

  /* USER CODE END ADC1_Init 2 */

}

/**
  * @brief I2C1 Initialization Function
  * @param None
  * @retval None
  */
static void MX_I2C1_Init(void)
{

  /* USER CODE BEGIN I2C1_Init 0 */

  /* USER CODE END I2C1_Init 0 */

  /* USER CODE BEGIN I2C1_Init 1 */

  /* USER CODE END I2C1_Init 1 */
  hi2c1.Instance = I2C1;
  hi2c1.Init.ClockSpeed = 100000;
  hi2c1.Init.DutyCycle = I2C_DUTYCYCLE_2;
  hi2c1.Init.OwnAddress1 = 0;
  hi2c1.Init.AddressingMode = I2C_ADDRESSINGMODE_7BIT;
  hi2c1.Init.DualAddressMode = I2C_DUALADDRESS_DISABLE;
  hi2c1.Init.OwnAddress2 = 0;
  hi2c1.Init.GeneralCallMode = I2C_GENERALCALL_DISABLE;
  hi2c1.Init.NoStretchMode = I2C_NOSTRETCH_DISABLE;
  if (HAL_I2C_Init(&hi2c1) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN I2C1_Init 2 */

  /* USER CODE END I2C1_Init 2 */

}

/**
  * @brief SPI1 Initialization Function
  * @param None
  * @retval None
  */
static void MX_SPI1_Init(void)
{

  /* USER CODE BEGIN SPI1_Init 0 */

  /* USER CODE END SPI1_Init 0 */

  /* USER CODE BEGIN SPI1_Init 1 */

  /* USER CODE END SPI1_Init 1 */
  /* SPI1 parameter configuration*/
  hspi1.Instance = SPI1;
  hspi1.Init.Mode = SPI_MODE_MASTER;
  hspi1.Init.Direction = SPI_DIRECTION_2LINES;
  hspi1.Init.DataSize = SPI_DATASIZE_8BIT;
  hspi1.Init.CLKPolarity = SPI_POLARITY_LOW;
  hspi1.Init.CLKPhase = SPI_PHASE_1EDGE;
  hspi1.Init.NSS = SPI_NSS_SOFT;
  hspi1.Init.BaudRatePrescaler = SPI_BAUDRATEPRESCALER_4;
  hspi1.Init.FirstBit = SPI_FIRSTBIT_MSB;
  hspi1.Init.TIMode = SPI_TIMODE_DISABLE;
  hspi1.Init.CRCCalculation = SPI_CRCCALCULATION_DISABLE;
  hspi1.Init.CRCPolynomial = 10;
  if (HAL_SPI_Init(&hspi1) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN SPI1_Init 2 */

  /* USER CODE END SPI1_Init 2 */

}

/**
  * Enable DMA controller clock
  */
static void MX_DMA_Init(void)
{

  /* DMA controller clock enable */
  __HAL_RCC_DMA1_CLK_ENABLE();

  /* DMA interrupt init */
  /* DMA1_Channel1_IRQn interrupt configuration */
  HAL_NVIC_SetPriority(DMA1_Channel1_IRQn, 0, 0);
  HAL_NVIC_EnableIRQ(DMA1_Channel1_IRQn);

}

/**
  * @brief GPIO Initialization Function
  * @param None
  * @retval None
  */
static void MX_GPIO_Init(void)
{
  GPIO_InitTypeDef GPIO_InitStruct = {0};
/* USER CODE BEGIN MX_GPIO_Init_1 */
/* USER CODE END MX_GPIO_Init_1 */

  /* GPIO Ports Clock Enable */
  __HAL_RCC_GPIOC_CLK_ENABLE();
  __HAL_RCC_GPIOD_CLK_ENABLE();
  __HAL_RCC_GPIOA_CLK_ENABLE();
  __HAL_RCC_GPIOB_CLK_ENABLE();

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(LED_PTT_GPIO_Port, LED_PTT_Pin, GPIO_PIN_RESET);

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(GPIOB, A_CODE_Pin|B_CODE_Pin|LCD_CS_Pin|LCD_RST_Pin
                          |LCD_DC_Pin|LED_DISP_Pin|C_CODE_Pin|D_CODE_Pin
                          |PE_V4_Pin|PE_V5_Pin|PE_V6_Pin, GPIO_PIN_RESET);

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(GPIOA, PE_V1_Pin|PE_V2_Pin|PE_V3_Pin, GPIO_PIN_RESET);

  /*Configure GPIO pin : LED_PTT_Pin */
  GPIO_InitStruct.Pin = LED_PTT_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(LED_PTT_GPIO_Port, &GPIO_InitStruct);

  /*Configure GPIO pins : A_CODE_Pin B_CODE_Pin LCD_CS_Pin LCD_RST_Pin
                           LCD_DC_Pin LED_DISP_Pin C_CODE_Pin D_CODE_Pin
                           PE_V4_Pin PE_V5_Pin PE_V6_Pin */
  GPIO_InitStruct.Pin = A_CODE_Pin|B_CODE_Pin|LCD_CS_Pin|LCD_RST_Pin
                          |LCD_DC_Pin|LED_DISP_Pin|C_CODE_Pin|D_CODE_Pin
                          |PE_V4_Pin|PE_V5_Pin|PE_V6_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);

  /*Configure GPIO pins : C_EXT_Pin D_EXT_Pin */
  GPIO_InitStruct.Pin = C_EXT_Pin|D_EXT_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
  GPIO_InitStruct.Pull = GPIO_PULLDOWN;
  HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);

  /*Configure GPIO pins : PE_V1_Pin PE_V2_Pin PE_V3_Pin */
  GPIO_InitStruct.Pin = PE_V1_Pin|PE_V2_Pin|PE_V3_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);

  /*Configure GPIO pins : A_EXT_Pin B_EXT_Pin */
  GPIO_InitStruct.Pin = A_EXT_Pin|B_EXT_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
  GPIO_InitStruct.Pull = GPIO_PULLDOWN;
  HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);

  /*Configure GPIO pin : PTT_ON_Pin */
  GPIO_InitStruct.Pin = PTT_ON_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_IT_RISING_FALLING;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  HAL_GPIO_Init(PTT_ON_GPIO_Port, &GPIO_InitStruct);

  /* EXTI interrupt init*/
  HAL_NVIC_SetPriority(EXTI15_10_IRQn, 0, 0);
  HAL_NVIC_EnableIRQ(EXTI15_10_IRQn);

/* USER CODE BEGIN MX_GPIO_Init_2 */
/* USER CODE END MX_GPIO_Init_2 */
}

/* USER CODE BEGIN 4 */

/* USER CODE END 4 */

/**
  * @brief  This function is executed in case of error occurrence.
  * @retval None
  */
void Error_Handler(void)
{
  /* USER CODE BEGIN Error_Handler_Debug */
  /* User can add his own implementation to report the HAL error return state */
  __disable_irq();
  while (1)
  {
  }
  /* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t *file, uint32_t line)
{
  /* USER CODE BEGIN 6 */
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */
